@@ -613,6 +613,7 @@ multisom.batch <-function(data= NULL,xheight, xwidth,
       }
     }
 
+
     nwithin1 <- length(within.dist1)
     nbetween1 <- length(between.dist1)
     meanwithin1 <- mean(within.dist1)
@@ -1151,6 +1152,11 @@ multisom.batch <-function(data= NULL,xheight, xwidth,
 
   n1<-n2<-n3<-n4<-n5<-n6<-n7<-n8<-n9<-n10<-n11<-n12<-n13<-n14<-n15<-n16<-n17<-
     n18<-n19<-n20<-n21<-n22<-n23<-n24<-n25<-n26<-n27<-n28<-n29<-n30<-1
+  data1 <- data
+  lis2 <- list()
+  lis3 <- list()
+  cf <- rep(0,nrow(data1))
+  l <- 1
 
   ###################################################################################
   repeat {
@@ -1158,6 +1164,7 @@ multisom.batch <-function(data= NULL,xheight, xwidth,
     res.batch <- BatchSOM(data,grid = somgrid(xheight, xwidth,topo), min.radius ,max.radius,maxit,init,radius.type)
     codes <- res.batch$code
     cl  <- res.batch$classif
+    xf <- cl
     Dist <- dist(data, method="euclidean")
 
     ##################################### Indice.db###################################
@@ -1331,14 +1338,55 @@ multisom.batch <-function(data= NULL,xheight, xwidth,
     ######################## Indice.hubert########################################
     if(is.na(match(22,indice))==FALSE||is.na(match(31,indice))==FALSE)
     {
-      result[n22,22] <-Indice.hubert(data, cl)
       lis1[[n22]] <- cl
+      lis2[[l]] <- cl
+      l <- l+1
+      if (length(lis2) > 1){
+        x <- xheight* xwidth
+        for(i in 1:x){
+          y <- i
+          j <- length(lis2)
+          repeat{
+            cl1 <- lis2[[j]]
+            if (length(y)== 1){
+              v <- which(cl1 == y)
+              y1 <- v
+            }
+            if (length(y) >= 1){
+              m <- 1
+              lis3 <- list()
+              for(l in 1:length(y)){
+                lis3[[m]] <- which(cl1== y[l])
+                m <- m+1
+              }
+              y1 <- unlist(lis3)
+            }
+            j <- j- 1
+            y <- y1
+            if( (j <= 0) | (length(y)==0))
+              break
+          }
+
+          if (length(y) !=0){
+            for( p in 1:length(y))
+            {
+              n <- y[p]
+              cf[n] <-i
+            }
+          }
+
+        }
+        cl <- cf
+
+      }
+      result[n22,22] <- Indice.hubert(data1, cl)
       n22 <-n22+1
     }
 
     ########################### Indice.sv#########################################
     if(is.na(match(23,indice))==FALSE||is.na(match(31,indice))==FALSE)
     {
+      cl <- xf
       result[n23,23]<-Indice.sv(cl,data)
       lis1[[n23]] <- cl
       n23 <-n23+1
@@ -1421,6 +1469,7 @@ multisom.batch <-function(data= NULL,xheight, xwidth,
     lis[[i]]<-unique(lis1[[i]])
   }
 
+  n <- max_nc-1
   dim <- rep(0,n)
   vide <-a <- a1 <-list()
   j <- j1 <-1
@@ -1674,7 +1723,7 @@ multisom.batch <-function(data= NULL,xheight, xwidth,
           Diff[n-min_nc+1,9] <- abs(result[n-min_nc+1,19]-result[n-min_nc,19])
           Diff[n-min_nc+1,10]<- abs(result[n-min_nc+1,22]-result[n-min_nc,22])
           Diff[n-min_nc+1,11]<- abs(result[n-min_nc+1,25]-result[n-min_nc,25]) # Hartigan
-          Diff[n-min_nc+1,12]<- abs(result[n-min_nc+1,27]-result[n-min_nc,27])   #xu
+          Diff[n-min_nc+1,12]<- ((result[n-min_nc+2,27]-result[n-min_nc+1,27])-(result[n-min_nc+1,27]-result[n-min_nc,27]))   #xu
 
         }
       }
@@ -1682,6 +1731,7 @@ multisom.batch <-function(data= NULL,xheight, xwidth,
 
   }
 
+
   nc.Scott<-indice.Scott<-0
   if(is.na(match(13,indice))==FALSE||is.na(match(31,indice))==FALSE)
   {
@@ -1982,6 +2032,8 @@ multisom.batch <-function(data= NULL,xheight, xwidth,
     x <-list()
     b <- list()
     num <- ne[[pos]]
+
+
     for(i in 1:nb[[pos]])
     {
       y <- which(lis1[[pos]]==num[i])
@@ -2016,6 +2068,8 @@ multisom.batch <-function(data= NULL,xheight, xwidth,
       }
     }
 
+
+
     ###############################################################
     grid = somgrid(max_nc,max_nc,topo)
     plot(grid)
@@ -2087,6 +2141,3 @@ multisom.batch <-function(data= NULL,xheight, xwidth,
   return(results.final)
 
   }
-
-
-
